<?php
/***
*	Classe : TForm
*
*   Notes : La classe TForm est la classe qui gre la gnration des formulaires. C'est celle qu'il faut instancier dans tous les cas.
*
*   Licence : Ce code est entirement libre de droits. Vous pouvez le copier, le modifier, le partager, et ce sans aucune restriction
*
*   Auteur : Christophe TAFANI-DEREEPER <christophetd@gmail.com>
*
*   Date de cration : 24 juillet 2009
*   Date de dernire modification : 20 aot 2009
* 
*/

class TForm
{
	/***
	* Le nom unique du formulaire
	* @var protected string nom
	*/
	protected $nom;
	
	/***
	* La mthode d'envoi des donnes du formulaire (POST ou GET)
	* @var protected string methode
	*/
	protected $methode;
	
	/***
	* La page de traitement du formulaire. Par dfaut, aucune action n'est effectue si le fichier n'existe pas.
	* Modifier le paramtre $checkAction du constructeur pour.
	* @var protected string nom
	*/
	protected $action;
	
	/***
	* Indique si les donnes sortantes du formulaire doivent tre traites par htmlspecialchars()
	* @var protected bool protectHtml
	*
	*/
	protected $protectHtml;
	
	/***
	* L'array contenant les objets, lments du formulaires, instancis depuis la mthode add()
	* @var protected array elements
	*/
	private $elements = array();
	

	/***
	* __construct
	*
	* Le constructeur de la classe ZForm
	*
	* @var string $nom = NULL				Le nom unique du formulaire
	* @var string $methode = POST			La mthode utilise par le formulaire pour transfrer les donnes
	* @var string $action = NULL			La page de traitement du formulaire
	* @var bool $checkAction = FALSE		Le bolen indiquant si le constructeur doit afficher une erreur en cas d'indication d'une page de traitement inexistante
	*/
	public function __construct($nom=NULL, $methode = 'POST', $action = NULL, $checkAction = false)
	{
		$this->nom = $nom;
		$this->methode = $methode;
		
		if($checkAction && !file_exists($action))
		{
			trigger_error('La page de traitement dfinie n\'existe pas.', E_USER_WARNING);
		}
		$this->action = $action;
		
		if(get_magic_quotes_gpc())
		{
			$_POST = array_map('stripslashes', $_POST);
			$_GET = array_map('stripslashes', $_GET);
			$_COOKIE = array_map('stripslashes', $_COOKIE);
		}
		
		$this->protectHtml = true;
	}

	/**
	* __toString()
	*
	* Gnre le formulaire lors d'un echo de l'instance de la classe
	*
	* @return string       Contient le html du formulaire gnr
	*/
	public function __toString()
	{
		if(empty($this->elements)) return 'Aucun lement dans le formulaire';
		
		$out = '';
		$out .= "\n<p>\n";
		$out .= '<form action="'.$this->action.'" method="'.$this->methode.'" id="'.$this->nom.'" enctype="multipart/form-data">';
		foreach($this->elements as $element)
		{
			$out .= $element->getHtml()."<br />\n";
		}
		$out .= '<br /><input type="submit" name="submit" />'."\n".'<br /></form>'."\n".'</p>';
		return $out;
	}
	
	/**
	* add($type)
	*
	*  Ajoute un champ en instanciant la classe correspondante et stocke l'objet instanci dans le tableau elements
	*
	* @var string $type					   Indique la nature du champ  ajouter
	* @return mixed 					   Object si le type est reconnu par la classe, bool (FALSE) dans le cas contraire
	*/
	public function add($type)
	{
		$rempls = array('Text' => 'TFTextInput', 
						'Email' => 'TFMailInput', 
						'Date' => 'TFDateInput', 
						'Checkbox' => 'TFCheckBox',
						'Radio' => 'TFRadio',
						'Select' => 'TFSelectList',
						'Password' => 'TFPasswordInput',
						'Hidden' => 'TFHiddenField', 
						'Textarea' => 'TFTextareaInput', 
						'File' => 'TFileInput');
		foreach($rempls as $i => $a)
		{
			$type = preg_replace('#^'.$i.'$#isU', $a, $type);
		}
		if(!($obj = new $type())) return false;
		$this->elements[] = $obj;
		return $obj;
	}
	
	/**
	* 
	*  Renvoie TRUE si le formulaire a t soumis, FALSE dans le cas contraire
	*
	* @return bool
	*/
	public function isSubmitted()
	{
		return (isset($_POST['submit']));
	}
	
	/**
	* 
	*  Renvoie TRUE si le formulaire a t soumis et s'il est conforme aux restrictions imposes par l'utilisateur
	*
	* @var array $array		L'array dans lequel chercher et vrifier la validit des valeurs (le plus souvent $_POST ou $_GET)
	* @return bool
	*/
	public function isValid($array)
	{
		if(empty($this->elements) OR !$this->isSubmitted()) return FALSE;
		
		foreach($this->elements as $obj)
		{
			if(!$obj->isValid($array))
			{
				return FALSE;
			}
		}
		
		return TRUE;
	}
	
	/**
	* Modifie l'attribut indiquant si la protection contre le HTML est active ou non
	*
	* @var bool $protect
	* @return bool
	*/
	public function protectHtml($protect)
	{
		return ($this->protectHtml = (bool) $protect);
	}
	
	/**
	* 
	* Appelle la mthode getError de chaque objet TFormElement stock dans le tableau element
	*
	* @return string		Les erreurs rencontres  la soumission du formulaire
	*/
	public function errors()
	{
		$out = '';
		foreach($this->elements as $obj)
		{
			$out .= $obj->getError();
		}
		return $out;
	}
	
	/**
	*
	* Pr-rempli les champs du formulaires avec les donnes de l'array
	* @var array array		L'array d'o les valeurs sortent (le plus souvent $_POST ou $_GET)
	*/
	public function pre(Array $array)
	{
		foreach($this->elements as $obj)
		{
			$var = @$array[$obj->getName()];
			if(isset($var))
			{
				$obj->value($var);
				$obj->defaultChoice($var);
			}
		}
	}
		
	/** 
	*
	* Appelle la mthode getError de chaque objet TFormElement stock dans le tableau element
	*
	* @multiple var string/array 		Les noms des champs dont la valeur est  rcuprer
	* @return array/string					Array des valeurs concernes, string si une seule valeur demande
	*/
	public function get_cleaned_data($param)
	{
		$cleaned_data = array();
		$out = '';
		
		if($this->protectHtml)
		{
			$_POST = array_map('htmlspecialchars', $_POST);
			$_GET = array_map('htmlspecialchars', $_GET);
		}
		
		if(is_array($param) && func_num_args()==1)
		{
			foreach($param as $name)
			{
				$cleaned_data[] = (isset($_POST[$name])) ? $_POST[$name] : NULL;
			}
		}
		elseif(!is_array($param) && func_num_args()==1)
		{
			return $_POST[$param];
		}
		else
		{
			foreach(func_get_args() as $name)
			{
				$cleaned_data[] = (isset($_POST[$name])) ? $_POST[$name] : NULL;
			}
		}
		return $cleaned_data;
	}
}